@@ -6,7 +6,7 @@
 #' @name base_pref
 #' @param expr A numerical/logical expression which is the term to evaluate for the current preference. 
 #'       The objective is to search for minimal/maximal values of this expression (for \code{low}/\code{high}) or for 
-#'       logical true values (for \code{true}). 
+#'       logical \code{TRUE} values (for \code{true}). 
 #' 
 #' @details
 #' 
@@ -20,18 +20,38 @@
 #'         The values of \code{a} must be numeric values.}
 #'   \item{\code{true(a)}}{Search for true values in logical expressions, i.e., \code{TRUE} is considered to be better than \code{FALSE}.
 #'         The values of \code{a} must be logical values.
-#'         For a tuplewise evaulation of a complex boolean condition one has to use the \code{&} and \code{|} operators for logical AND/OR
+#'         For a tuplewise evaluation of a complex logical expression one has to use the \code{&} and \code{|} operators for logical AND/OR
 #'         (and not the \code{&&} and \code{||} operators).}
 #' }
 #' 
 #' 
-#' The term \code{expr} can be just a single attribute or an arbitrary expression, e.g., \code{low(a+2*b+f(c))}.
+#' The term \code{expr} may be just a single attribute or may contain an arbitrary expression, e.g., \code{low(a+2*b+f(c))}.
 #' There \code{a}, \code{b} and \code{c} are columns of the addressed dataset and \code{f} is a previously defined function.
 #' 
 #' Functions contained in \code{expr} are evaluated over the entire dataset, i.e., 
 #' it is possible to use aggregate functions (\code{min}, \code{mean}, etc.). 
 #' Note that all functions (and also variables which are not columns of the dataset, where \code{expr} will be evaluated on)
-#' must be defined in the same environment (e.g. environment of a function) as the base preference.
+#' must be defined in the same environment (e.g. environment of a function or global scope) as the base preference.
+#' 
+#' @section Using expressions in preferences:
+#' 
+#' The \code{low_(e)}, \code{high_(e)} and \code{true_(e)} preferences have the same functionality,
+#' but expect an expression \code{e} or symbol \code{e} as argument.
+#' For example, \code{low(a)} is equivalent to \code{low(expression(a))} or \code{low(as.symbol("a"))}. 
+#' 
+#' 
+#' This is very helpful for developing your own base preferences. Assume you want to define a base Preference \code{false}
+#' as the dual of \code{true}. A definition like \code{false <- function(x) -true(x)} is the wrong approach, as 
+#' \code{psel(data.frame(a = c(1,2)), false(a == 1))} will result in the error \code{object 'a' not found}.
+#' This is because \code{a} is considered as a variable and not as an (abstract) symbol to be evaluated later on the data set.
+#' By defining
+#' 
+#' \code{false <- function(x) -true_(substitute(x))}
+#' 
+#' one gets a preference which behaves like a "built-in" preference. The object \code{false(a == 1)} will output 
+#' \code{[Preference] -true(a == 1)} on the console and 
+#' \code{psel(data.frame(a = c(1,2)), false(a==1))} returns correctly the second tuple with \code{a==2}.
+#' 
 #' 
 #' @seealso See \code{\link{complex_pref}} how to compose complex preferences to retrieve e.g. the Skyline.
 #' 
@@ -57,6 +77,12 @@ low <- function(expr) {
   return(lowpref(expr, parent.frame()))
 }
 
+#' @rdname base_pref
+#' @export
+low_ <- function(expr) {
+  return(lowpref(as.expression(expr), parent.frame()))
+}
+
 #' @rdname base_pref
 #' @export
 high <- function(expr) {
@@ -64,9 +90,21 @@ high <- function(expr) {
   return(highpref(expr, parent.frame()))
 }
 
+#' @rdname base_pref
+#' @export
+high_ <- function(expr) {
+  return(highpref(as.expression(expr), parent.frame()))
+}
+
 #' @rdname base_pref
 #' @export
 true <- function(expr) {
   expr <- as.expression(substitute(expr))
   return(truepref(expr, parent.frame()))
 }
+
+#' @rdname base_pref
+#' @export
+true_ <- function(expr) {
+  return(truepref(as.expression(expr), parent.frame()))
+}
\ No newline at end of file
