@@ -4,9 +4,11 @@
 #' 
 #' @name pred_succ
 #' 
-#' @param df A dataframe characterizing the basic set wherein predecessors/sucessors are seached.
-#' @param v A numeric vector of indices in df. This describes the set of tuples for which predecessors/successors are searched.
+#' @param df A dataframe characterizing the set wherein predecessors/sucessors are searched.
+#' @param v A numeric vector of indices in df. The represents the set of tuples for which predecessors/successors are searched.
 #' @param p A preference. Worse tuples in the induced order are succesors and better tuples are predecessors.
+#' @param intersect Logical value. If \code{FALSE} (by default) the union of all predecessors/successors of \code{v} are returned.
+#'        For \code{intersect = TRUE} the intersection of those is returned.
 #' 
 #' @details
 #' 
@@ -17,17 +19,29 @@
 #' 
 #' There \code{p} is a preference object and \code{df} a dataframe. This statement calculates the Better-Than-Relation on \code{df} w.r.t. \code{p}. 
 #' Afterwards the subsequent predecessor and sucessor functions can be called. 
-#' The return values and the values of \code{v} are subsets of \code{1:nrow(df)} and characerise a subset of tuples in \code{df}. 
+#' The value of \code{v} is a numeric vectors within \code{1:nrow(df)} 
+#' and characerizes a subset of tuples in \code{df}. 
+#' The return value of these functions is again a numeric vector refering to the row numbers in \code{df} 
+#' and it is always ordered ascending, independent of the order of the indices in \code{v}.
+#' 
 #' 
 #' \describe{
 #'   \item{\code{all_pred(p, v)}}{Returns all predecessors of \code{v}, i.e. indices of better tuples than \code{v}.}
 #'   \item{\code{all_succ(p, v)}}{Returns all predecessors of \code{v}, i.e. indices of worse tuples than \code{v}.}
 #'   \item{\code{hasse_pred(p, v)}}{Returns the direct predecessors of \code{v}, 
-#'         i.e. indices of better tuples than \code{v} where the Better-Than-Relation lies in the transitive reduction.}
+#'         i.e. indices of better tuples than \code{v} where the Better-Than-Relation is contained in the transitive reduction.}
 #'   \item{\code{hasse_succ(p, v)}}{Returns the direct successors of \code{v}, 
-#'         i.e. indices of worse tuples than \code{v} where the Better-Than-Relation lies in the transitive reduction.}
+#'         i.e. indices of worse tuples than \code{v} where the Better-Than-Relation is contained in the transitive reduction.}
 #' }
 #' 
+#' If \code{v} has length 1, then the value of \code{intersect} does not matter, as there is nothing to intersect or join. 
+#' For scalar values \code{x} and \code{y} the following identities hold, where \code{f} is one of the predecessor/successor functions:
+#' 
+#' \code{f(p, c(x, y), intersect = FALSE) == union(f(p, x), f(p, y))}
+#' 
+#' \code{f(p, c(x, y), intersect = TRUE) == intersect(f(p, x), f(p, y))}
+#' 
+#' 
 #' @examples
 #' 
 #' # Preference on mtcars for high mpg and low weight
@@ -40,7 +54,7 @@
 #' # Pick some tuple "in the middle"
 #' show_vals(10)
 #' 
-#' # Show (direct) predecessors/successors
+#' # Show (direct) predecessors/successors of tuple 10
 #' show_vals(hasse_pred(p, 10)) # Next better car
 #' show_vals(hasse_succ(p, 10)) # Next worse car
 #' show_vals(all_pred(p, 10))   # All better cars
@@ -62,24 +76,24 @@ init_pred_succ <- function(df, p) {
 
 #' @rdname pred_succ
 #' @export
-hasse_pred <- function(p, v) {
-  p$h_pred(v)
+hasse_pred <- function(p, v, intersect = FALSE) {
+  p$h_predsucc(v, do_intersect = intersect, succ = FALSE)
 }
 
 #' @rdname pred_succ
 #' @export
-hasse_succ <- function(p, v) {
-  p$h_succ(v)
+hasse_succ <- function(p, v, intersect = FALSE) {
+  p$h_predsucc(v, do_intersect = intersect, succ = TRUE)
 }
 
 #' @rdname pred_succ
 #' @export
-all_pred <- function(p, v) {
-  p$all_pred(v)
+all_pred <- function(p, v, intersect = FALSE) {
+  p$all_predsucc(v, do_intersect = intersect, succ = FALSE)
 }
 
 #' @rdname pred_succ
 #' @export
-all_succ <- function(p, v) {
-  p$all_succ(v)
+all_succ <- function(p, v, intersect = FALSE) {
+  p$all_predsucc(v, do_intersect = intersect, succ = TRUE)
 }
